﻿-- **********************************************************************
-- GnomTEC Gnomcorder Fight
-- Version: 7.0.3.11
-- Author: GnomTEC
-- Copyright 2011-2016 by GnomTEC
-- http://www.gnomtec.de/
-- **********************************************************************
-- load localization first.
local L = LibStub("AceLocale-3.0"):GetLocale("GnomTEC_Gnomcorder_Fight")

-- ----------------------------------------------------------------------
-- Legacy global variables and constants (will be deleted in future)
-- ----------------------------------------------------------------------

-- ----------------------------------------------------------------------
-- Addon global Constants (local)
-- ----------------------------------------------------------------------

-- Prefix for addon communication
local CONST_ADDONMESSAGE_PREFIX = "GNOMTEC_FIGHT"

-- regeneration times for char values in seconds
local CONST_REGENERATIONTIME_LP = 1800
local CONST_REGENERATIONTIME_AU = 25

-- constants for fight balancing
local CONST_AU_COST_AN = 2
local CONST_AU_COST_SAN = 2
local CONST_AU_COST_PA = 1


-- ----------------------------------------------------------------------
-- Addon global variables (local)
-- ----------------------------------------------------------------------

-- Main options menue with general addon information
local optionsMain = {
	name = "GnomTEC Gnomcorder Fight",
	type = "group",
	args = {
		descriptionTitle = {
			order = 1,
			type = "description",
			name = "Kampfsimulation für GnomTEC Gnomcorder.\n\n",
		},
		descriptionAbout = {
			name = "About",
			type = "group",
			guiInline = true,
			order = 2,
			args = {
				descriptionVersion = {
				order = 1,
				type = "description",			
				name = "|cffffd700".."Version"..": ".._G["GREEN_FONT_COLOR_CODE"]..GetAddOnMetadata("GnomTEC_Gnomcorder_Fight", "Version"),
				},
				descriptionAuthor = {
					order = 2,
					type = "description",
					name = "|cffffd700".."Autor"..": ".."|cffff8c00".."GnomTEC",
				},
				descriptionEmail = {
					order = 3,
					type = "description",
					name = "|cffffd700".."Email"..": ".._G["HIGHLIGHT_FONT_COLOR_CODE"].."info@gnomtec.de",
				},
				descriptionWebsite = {
					order = 4,
					type = "description",
					name = "|cffffd700".."Website"..": ".._G["HIGHLIGHT_FONT_COLOR_CODE"].."http://www.gnomtec.de/",
				},
				descriptionLicense = {
					order = 5,
					type = "description",
					name = "|cffffd700".."Copyright"..": ".._G["HIGHLIGHT_FONT_COLOR_CODE"].."(c)2011-2016 by GnomTEC",
				},
			}
		},
		descriptionLogo = {
			order = 5,
			type = "description",
			name = "",
			image = "Interface\\AddOns\\GnomTEC_Gnomcorder\\Textures\\GnomTEC-Logo",
			imageCoords = {0.0,1.0,0.0,1.0},
			imageWidth = 512,
			imageHeight = 128,
		},
	}
}


-- Last time when a timer event was triggerd
local lastTimerEvent = GetTime()
local lastTimerEventLP = lastTimerEvent
local lastTimerEventAU = lastTimerEvent


-- variables for the actual values of "lebenspunkte" und "Ausdauer"
local actualLP = 0
local actualAU = 0

-- variables for the target players values of "lebenspunkte" und "Ausdauer"
local targetPlayer = ""
local targetLP = 0
local targetMaxLP = 1
local targetAU = 0
local targetMaxAU = 1

-- ----------------------------------------------------------------------
-- Startup initialization
-- ----------------------------------------------------------------------

GnomTEC_Gnomcorder_Fight = LibStub("AceAddon-3.0"):NewAddon("GnomTEC_Gnomcorder_Fight", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0")
local Tourist = LibStub("LibTourist-3.0") 

LibStub("AceConfig-3.0"):RegisterOptionsTable("GnomTEC Gnomcorder Fight Main", optionsMain)
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GnomTEC Gnomcorder Fight Main", "GnomTEC Gnomcorder Fight");

-- ----------------------------------------------------------------------
-- Local functions
-- ----------------------------------------------------------------------

-- function which returns also nil for empty strings
local function emptynil( x ) return x ~= "" and x or nil end

function GnomTEC_Gnomcorder_Fight:CheckEquipment()
	-- works only for german and english clients at the moment because of missing localization
	local itemIDChest = GetInventoryItemID("player", GetInventorySlotInfo("ChestSlot"))
	local itemIDMainHand = GetInventoryItemID("player", GetInventorySlotInfo("MainHandSlot"))
	local itemIDSecondaryHand = GetInventoryItemID("player", GetInventorySlotInfo("SecondaryHandSlot"))
	local itemIDRanged = nil

	-- initialize values to naked defaults
	self.db.char.characterValues.RS_EQUIPMENT = 0
	self.db.char.characterValues.RB_EQUIPMENT = 0
	self.db.char.characterValues.PA_EQUIPMENT = 0
	self.db.char.characterValues.AT_EQUIPMENT = 0
	self.db.char.characterValues.W_AT_EQUIPMENT = 1
	self.db.char.characterValues.SAT_EQUIPMENT = 0
	self.db.char.characterValues.W_SAT_EQUIPMENT = 1
	
	if (itemIDChest) then
		local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemIDChest)

		if (sSubType == "Cloth") or (sSubType == "Stoff") then
			self.db.char.characterValues.RS_EQUIPMENT = 0
			self.db.char.characterValues.RB_EQUIPMENT = 0
			self.db.char.characterValues.PA_EQUIPMENT = 0
		elseif (sSubType == "Leather") or (sSubType == "Leder") then
			self.db.char.characterValues.RS_EQUIPMENT = 1
			self.db.char.characterValues.RB_EQUIPMENT = 1
			self.db.char.characterValues.PA_EQUIPMENT = -1
		elseif (sSubType == "Mail") or (sSubType == "Kette") then
			self.db.char.characterValues.RS_EQUIPMENT = 2
			self.db.char.characterValues.RB_EQUIPMENT = 2
			self.db.char.characterValues.PA_EQUIPMENT = -2
		elseif (sSubType == "Plate") or (sSubType == "Platte") then
			self.db.char.characterValues.RS_EQUIPMENT = 3
			self.db.char.characterValues.RB_EQUIPMENT = 3
			self.db.char.characterValues.PA_EQUIPMENT = -3
		end
	end

	if (itemIDMainHand) then
		local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemIDMainHand)

		if (sType == "Weapon") or (sType == "Waffe") then	
			if (sSubType == "One-Handed Swords") or (sSubType == "Einhandschwerter") then	
				self.db.char.characterValues.W_AT_EQUIPMENT = 6
				self.db.char.characterValues.AT_EQUIPMENT = 0				
			end
		end
	end

	if (itemIDSecondaryHand) then
		local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemIDSecondaryHand)
		if (sType == "Weapon") or (sType == "Waffe") then
			-- not supported yet
		elseif (sSubType == "Shields") or (sSubType == "Schilde") then
			self.db.char.characterValues.PA_EQUIPMENT = self.db.char.characterValues.PA_EQUIPMENT + 2
		end
	end

	if (itemIDRanged) then
		local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemIDRanged)

		if (sType == "Weapon") or (sType == "Waffe") then	
			if (sSubType == "Guns") or (sSubType == "Schusswaffen") then	
				self.db.char.characterValues.W_SAT_EQUIPMENT = 6
				self.db.char.characterValues.SAT_EQUIPMENT = 3				
			end
		end

	end

end


-- function computes and displays character values from attributes, armor and weapons
function GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
	local mu = self.db.char.characterAttributes.MU
	local st =	self.db.char.characterAttributes.ST
	local will = self.db.char.characterAttributes.WILL
	local intu = self.db.char.characterAttributes.INTU
	local bew =	self.db.char.characterAttributes.BEW
	local gp =	self.db.char.characterAttributes.GP
	local ap =	self.db.char.characterAttributes.AP
	local lp_add = self.db.char.characterValues.LP_ADD
	local au_add = self.db.char.characterValues.AU_ADD
	local an_add = self.db.char.characterValues.AN_ADD
	local pa_add = self.db.char.characterValues.PA_ADD
	local san_add = self.db.char.characterValues.SAN_ADD
	local rs_add = self.db.char.characterValues.RS_ADD
	local rb_sub = self.db.char.characterValues.RB_SUB
	local lp = floor((st + st + will)/4)
	local au = floor((mu + will + bew)/3)
	local an = floor((mu + st + bew)/5)
	local pa = floor((intu + bew + st)/5)
	local san = floor((bew +  mu + intu)/6)
	local rs = self.db.char.characterValues.RS_EQUIPMENT
	local rb = self.db.char.characterValues.RB_EQUIPMENT
	local pa_equipment = self.db.char.characterValues.PA_EQUIPMENT
	
	self.db.char.characterValues.LP = lp
	self.db.char.characterValues.AU = au
	self.db.char.characterValues.AN = an
	self.db.char.characterValues.PA = pa
	self.db.char.characterValues.SAN = san
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU:SetText(mu)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST:SetText(st)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL:SetText(will)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU:SetText(intu)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW:SetText(bew)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_GP:SetText(gp)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_AP:SetText(ap)
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU_BUTTON:SetText(mu)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST_BUTTON:SetText(st)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL_BUTTON:SetText(will)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU_BUTTON:SetText(intu)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW_BUTTON:SetText(bew)
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_LP:SetText(actualLP.." / "..lp.."|cff10FF10+"..lp_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_AU:SetText(actualAU.." / "..au.."|cff10FF10+"..au_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_AN:SetText(an.."|cff10FF10+"..an_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_PA:SetText(pa.."|cffFFFF10+"..pa_equipment.."|cff10FF10+"..pa_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_SAN:SetText(san.."|cff10FF10+"..san_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_RS:SetText("|cffFFFF10"..rs.."|cff10FF10+"..rs_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_VALUE_RB:SetText("|cffFFFF10"..rb.."|cff10FF10-"..rb_sub)	
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_NAME:SetText(UnitName("PLAYER"))
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_LP:SetMinMaxValues(0,lp+lp_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_LP:SetValue(actualLP)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_LP_TEXT:SetText(actualLP.."/"..lp+lp_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_LP_PERCENT:SetText(floor(100*actualLP/(lp+lp_add)).."%")
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_AU:SetMinMaxValues(0,au+au_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_AU:SetValue(actualAU)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_AU_TEXT:SetText(actualAU.."/"..au+au_add)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_AU_PERCENT:SetText(floor(100*actualAU/(au+au_add)).."%")

	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_NAME:SetText(targetPlayer)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_LP:SetMinMaxValues(0,targetMaxLP)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_LP:SetValue(targetLP)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_LP_TEXT:SetText(targetLP.."/"..targetMaxLP)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_LP_PERCENT:SetText(floor(100*targetLP/targetMaxLP).."%")
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_AU:SetMinMaxValues(0,targetMaxAU)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_AU:SetValue(targetAU)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_AU_TEXT:SetText(targetAU.."/"..targetMaxAU)
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_AU_PERCENT:SetText(floor(100*targetAU/targetMaxAU).."%")
end

-- set all attributes to default values
function GnomTEC_Gnomcorder_Fight:ResetCharacterAttributes()
	self.db.char.characterAttributes.SAVED = false
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_RESET:Hide();
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Show();	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Enable();
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW:Hide()

	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU_BUTTON:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST_BUTTON:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL_BUTTON:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU_BUTTON:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW_BUTTON:Show()
	
	self.db.char.characterAttributes.MU = 10
	self.db.char.characterAttributes.ST = 10
	self.db.char.characterAttributes.WILL = 10
	self.db.char.characterAttributes.INTU = 10
	self.db.char.characterAttributes.BEW = 10
	self.db.char.characterAttributes.GP = 0

	self.db.char.characterValues.LP_ADD = 0
	self.db.char.characterValues.AU_ADD = 0
	self.db.char.characterValues.AN_ADD = 0
	self.db.char.characterValues.PA_ADD = 0
	self.db.char.characterValues.SAN_ADD= 0
	self.db.char.characterValues.RS_ADD = 0
	self.db.char.characterValues.RB_SUB = 0

	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
	actualLP = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	actualAU = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
end

function GnomTEC_Gnomcorder_Fight:SaveCharacterAttributes()
	self.db.char.characterAttributes.SAVED = true
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Hide();
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_RESET:Show();
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU_BUTTON:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST_BUTTON:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL_BUTTON:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU_BUTTON:Hide()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW_BUTTON:Hide()
	
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU:Show()
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW:Show()

	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
	actualLP = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	actualAU = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
end

-- increments given attribute when GP available and not reached maximum
function GnomTEC_Gnomcorder_Fight:IncrementCharacterAttributes(attr)
	if (self.db.char.characterAttributes.GP > 0) then
		self.db.char.characterAttributes[attr] = (self.db.char.characterAttributes[attr] or 0) + 1
		self.db.char.characterAttributes.GP = self.db.char.characterAttributes.GP - 1
		GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
		actualLP = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
		actualAU = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
	end
end

-- dencrements given attribute when not already 0
function GnomTEC_Gnomcorder_Fight:DecrementCharacterAttributes(attr)
	if (self.db.char.characterAttributes[attr] > 0) then
		self.db.char.characterAttributes[attr] = self.db.char.characterAttributes[attr] - 1
		self.db.char.characterAttributes.GP = self.db.char.characterAttributes.GP + 1
		GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
		actualLP = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
		actualAU = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
	end
end

-- increments given value by using APs
function GnomTEC_Gnomcorder_Fight:UseAP(value)

	-- missing implementation!


	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
	actualLP = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	actualAU = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
end


-- callback function for timer events 
local function GnomTEC_Gnomcorder_Fight_TimerEvent()
	local t = GetTime()
		
	if ((t-lastTimerEvent) > 5) then
		-- every five seconds is quite enough
		if (UnitName("target") and (targetMaxLP > 1)) then
			-- target has addon so we can ask for data
			local lp = GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP + GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP_ADD
			local au = GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU + GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU_ADD

			GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "I:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")
		end
		
		if (actualLP >= (GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP + GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP_ADD)) then
			-- at maximum LP no regeneration
			lastTimerEventLP = t
		elseif ((t-lastTimerEventLP) > CONST_REGENERATIONTIME_LP) then
			actualLP = actualLP + 1
			lastTimerEventLP = t
		end

		if (actualAU >= (GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU + GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU_ADD)) then
			-- at maximum AU no regeneration
			lastTimerEventAU = t
		elseif ((t-lastTimerEventAU) > CONST_REGENERATIONTIME_AU) then
			actualAU = actualAU + 1
			lastTimerEventAU = t
		end

		GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
		lastTimerEvent = t
	end
end

-- ----------------------------------------------------------------------
-- Frame event handler and functions
-- ----------------------------------------------------------------------

-- initialize drop down menu
local function GnomTEC_Gnomcorder_Fight_UseAP_InitializeDropDown(level)
	local info = {}

	function info.func(arg1)
		GnomTEC_Gnomcorder_Fight:UseAP(arg1)
	end	
	info.colorCode = "|cff10FF10"
	info.notCheckable = true

	if (not GnomTEC_Gnomcorder_Fight.db) then
		info.disabled = true
		info.text = "<Noch nicht initialisiert>"	
	else
		local ap = GnomTEC_Gnomcorder_Fight.db.char.characterAttributes.AP
		local cost, y

		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP_ADD
		cost = ceil((y+1)*(20+((y+1)*0.1)))
		if (cost > ap) then
			info.disabled = true
		else
			info.disabled = false
		end	
		info.text = "Lebenspunkte +1 ("..cost.." AP)"
		info.arg1 = "LP_ADD"
		UIDropDownMenu_AddButton(info)
	
		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU_ADD
		cost = ceil((y+1)*(5+((y+1)*0.1)))
		if (cost > ap) then
			info.disabled = true
		else
			info.disabled = false
		end	
		info.text = "Ausdauer +1 ("..cost.." AP)"
		info.arg1 = "AU_ADD"
		UIDropDownMenu_AddButton(info)

		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.AN_ADD
		cost = ceil((y+1)*(4+((y+1)*0.1)))
		if (cost > ap) then
			info.disabled = true
		else
			info.disabled = false
		end	
		info.text = "Angriffswert +1 ("..cost.." AP)"
		info.arg1 = "AN_ADD"
		UIDropDownMenu_AddButton(info)

		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.PA_ADD
		cost = ceil((y+1)*(4+((y+1)*0.1)))
		if (cost > ap) then
			info.disabled = true
		else
			info.disabled = false
		end	
		info.text = "Parierwert +1 ("..cost.." AP)"
		info.arg1 = "PA_ADD"
		UIDropDownMenu_AddButton(info)

		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.SAN_ADD
		cost = ceil((y+1)*(4+((y+1)*0.1)))
		if (cost > ap) then
			info.disabled = true
		else
			info.disabled = false
		end	
		info.text = "Schußwaffenangriffswert +1 ("..cost.." AP)"
		info.arg1 = "SAN_ADD"
		UIDropDownMenu_AddButton(info)

		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.RS_ADD
		cost = ceil((y+1)*(4+((y+1)*0.1)))
		info.disabled = true
		info.text = "Rüstungsschutz +1 (? AP)"
		info.arg1 = "RS_ADD"
		UIDropDownMenu_AddButton(info)

		y = GnomTEC_Gnomcorder_Fight.db.char.characterValues.RB_SUB
		cost = ceil((y+1)*(4+((y+1)*0.1)))
		info.disabled = true
		info.text = "Rüstungsbehinderung -1 (? AP)"
		info.arg1 = "RB_SUB"
		UIDropDownMenu_AddButton(info)
	end
end

-- select UseAP drop down menu OnLoad
function GnomTEC_Gnomcorder_Fight:UseAP_DropDown_OnLoad(self)
	UIDropDownMenu_Initialize(self, GnomTEC_Gnomcorder_Fight_UseAP_InitializeDropDown, "MENU")
end

-- select UseMap drop down menu OnClick
function GnomTEC_Gnomcorder_Fight:UseAP_Button_OnClick(self, button, down)
	ToggleDropDownMenu(1, nil, GNOMTEC_GNOMCORDER_FIGHT_FRAME_VALUES_USEAP_DROPDOWN, self:GetName(), 0, 0)
end

function GnomTEC_Gnomcorder_Fight:GetTargetInformation()
	local player, realm = UnitName("target")

	if UnitIsPlayer("target") and player then
		local lp = GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP + GnomTEC_Gnomcorder_Fight.db.char.characterValues.LP_ADD
		local au = GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU + GnomTEC_Gnomcorder_Fight.db.char.characterValues.AU_ADD

		GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "I:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")	
	end
end


-- ----------------------------------------------------------------------
-- Hook functions
-- ----------------------------------------------------------------------

-- ----------------------------------------------------------------------
-- Event handler
-- ----------------------------------------------------------------------

-- event handler for addon communication messages
function GnomTEC_Gnomcorder_Fight:OnCommReceived(prefix, message, distribution, sender)
	if (prefix == CONST_ADDONMESSAGE_PREFIX) then
		local typeAT, valueAT, valueLP, maxLP, valueAU, maxAU	= strsplit(":",message);
		local dicePA = random(20)
		local output = nil
		
		valueAT = tonumber(valueAT) or 0
		valueLP = tonumber(valueLP) or 0
		maxLP = tonumber(maxLP) or 1
		valueAU = tonumber(valueAU) or 0
		maxAU = tonumber(maxAU) or 1
		
		
		local lp = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
		local au = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
		
		if (typeAT == "S") then
			-- a shooting attack
			if (CONST_AU_COST_PA <= actualAU) and (dicePA <= (self.db.char.characterValues.PA + self.db.char.characterValues.PA_EQUIPMENT + self.db.char.characterValues.PA_ADD - (self.db.char.characterValues.RB_EQUIPMENT - self.db.char.characterValues.RB_SUB))) then
				output = "pariert den Angriff von "..sender.." erfolgreich"
				actualAU = actualAU - CONST_AU_COST_PA
			elseif (valueAT <= (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD)) then
				output = "kann den Angriff von "..sender.." nicht parieren der Schaden wird aber von von der Rüstung abgefangen"
			else
				output = "kann den Angriff von "..sender.." nicht parieren und erhält Schaden *S"..(valueAT - (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD)).."*"
				if (actualLP > (valueAT - (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD))) then
					actualLP = actualLP - (valueAT - (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD))
				else
					actualLP = 0
				end
				GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
			end
			GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "R:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", sender , "NORMAL")
			if (sender == UnitName("TARGET")) then
				targetLP = valueLP
				targetMaxLP = maxLP
				targetAU = valueAU
				targetMaxAU = maxAU
				GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
			end
		elseif (typeAT == "A") then
			-- a normal attack
			if (CONST_AU_COST_PA <= actualAU) and (dicePA <= (self.db.char.characterValues.PA + self.db.char.characterValues.PA_EQUIPMENT + self.db.char.characterValues.PA_ADD - (self.db.char.characterValues.RB_EQUIPMENT - self.db.char.characterValues.RB_SUB))) then
				output = "pariert den Angriff von "..sender.." erfolgreich"
				actualAU = actualAU - CONST_AU_COST_PA
			elseif (valueAT <= (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD)) then
				output = "kann den Angriff von "..sender.." nicht parieren der Schaden wird aber von von der Rüstung abgefangen"
			else
				output = "kann den Angriff von "..sender.." nicht parieren und erhält Schaden *S"..(valueAT - (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD)).."*"
				if (actualLP > (valueAT - (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD))) then
					actualLP = actualLP - (valueAT - (self.db.char.characterValues.RS_EQUIPMENT + self.db.char.characterValues.RS_ADD))
				else
					actualLP = 0
				end
				GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
			end
			GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "R:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", sender , "NORMAL")
			if (sender == UnitName("TARGET")) then
				targetLP = valueLP
				targetMaxLP = maxLP
				targetAU = valueAU
				targetMaxAU = maxAU
				GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
			end
		elseif (typeAT == "I") then
			--  enemy is inspecting you to see your LP and AU
			GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "R:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", sender , "NORMAL")
			if (sender == UnitName("TARGET")) then
				targetLP = valueLP
				targetMaxLP = maxLP
				targetAU = valueAU
				targetMaxAU = maxAU
				GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
			end
		elseif (typeAT == "R") then
			-- a players reply with LP to your inspect or attack requests
			if (sender == UnitName("TARGET")) then
				targetLP = valueLP
				targetMaxLP = maxLP
				targetAU = valueAU
				targetMaxAU = maxAU
				GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
			end
		end

		if (output) then
			SendChatMessage(output,"EMOTE");
		end
	end
end

-- player want's to execute a attack with slash command
function GnomTEC_Gnomcorder_Fight:AttackEmote(input)
	local diceSAN = random(20)
	local diceAT = random(self.db.char.characterValues.W_AT_EQUIPMENT) + self.db.char.characterValues.AT_EQUIPMENT
	local output

	local lp = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	local au = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
	
	if UnitName("target") then
		if (CONST_AU_COST_AN > actualAU) then
			output = input .. " *Angriff auf "..UnitName("target").." ist zu schwach*"
			diceAT = 0		
			actualAU = 0
		elseif (diceSAN > self.db.char.characterValues.SAN) then
			output = input .. " *verfehlt "..UnitName("target").."*"
			diceAT = 0
			actualAU = actualAU - CONST_AU_COST_AN
		else	
			output = input .. " *AT"..diceAT.." "..UnitName("target").."*"
			actualAU = actualAU - CONST_AU_COST_AN
		end
	else
		output = "macht ein paar Übungsschläge"
		diceAT = 0
	end
	SendChatMessage(output,"EMOTE");
	if (diceAT > 0) then
		GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "A:"..diceAT..":"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")
	elseif UnitName("target") then 
		GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "I:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")
	end
	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
end

-- player want's to execute a shoot with slash command
function GnomTEC_Gnomcorder_Fight:ShootEmote(input)
	local diceAN = random(20)
	local diceAT = random(self.db.char.characterValues.W_SAT_EQUIPMENT) + self.db.char.characterValues.SAT_EQUIPMENT
	local output
	
	local lp = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	local au = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD

	if UnitName("target") then
		if (CONST_AU_COST_SAN > actualAU) then
			output = input .. " *Angriff auf "..UnitName("target").." ist zu schwach*"
			diceAT = 0
			actualAU = 0
		elseif (diceAN > self.db.char.characterValues.AN) then
			output = input .. " *verfehlt "..UnitName("target").."*"
			diceAT = 0
			actualAU = actualAU - CONST_AU_COST_SAN
		else	
			output = input .. " *AT"..diceAT.." "..UnitName("target").."*"
			actualAU = actualAU - CONST_AU_COST_SAN
		end
	else
		output = "schießt in die Luft"
		diceAT = 0
	end
	SendChatMessage(output,"EMOTE");
	if (diceAT > 0) then
		GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "S:"..diceAT..":"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")
	elseif UnitName("target") then 
		GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "I:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")
	end
	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
end

-- player want's to inspect with slash command
function GnomTEC_Gnomcorder_Fight:InspectEmote(input)

	local lp = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	local au = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD


	if UnitName("target") then
		output = input
		GnomTEC_Gnomcorder_Fight:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, "I:0:"..actualLP..":"..lp..":"..actualAU..":"..au, "WHISPER", UnitName("target") , "NORMAL")
	else
		output = "schaut in die Luft"
	end
	SendChatMessage(output,"EMOTE");
end

function GnomTEC_Gnomcorder_Fight:PLAYER_TARGET_CHANGED(eventName)
	local player, realm = UnitName("target")

	if UnitIsPlayer("target") and player then
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_PLAYERMODEL:SetUnit("target")
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_PLAYERMODEL:SetCamera(0)
		targetPlayer = player
		targetMaxLP = 1
		targetLP = 0
		targetMaxAU = 1
		targetAU = 0
		
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_PLAYERMODEL:SetUnit("player")
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_PLAYERMODEL:SetCamera(0)
	else
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_TARGET_PLAYERMODEL:ClearModel()
		targetPlayer = ""
		targetMaxLP = 1
		targetLP = 0
		targetMaxAU = 1
		targetAU = 0	
	end
	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
end

function GnomTEC_Gnomcorder_Fight:UNIT_INVENTORY_CHANGED(eventName, unitID)
	if (strupper(unitID) == "PLAYER") then
		GnomTEC_Gnomcorder_Fight:CheckEquipment()
		GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
	end
end

-- ----------------------------------------------------------------------
-- Addon OnInitialize, OnEnable and OnDisable
-- ----------------------------------------------------------------------

-- function called on initialization of addon
function GnomTEC_Gnomcorder_Fight:OnInitialize()
 	-- Code that you want to run when the addon is first loaded goes here.
	self.db = LibStub("AceDB-3.0"):New("GnomTEC_Gnomcorder_FightDB")

  	GnomTEC_Gnomcorder_Fight:Print("Willkommen bei GnomTEC_Gnomcorder_Fight")
  	  	
end

-- function called on enable of addon
function GnomTEC_Gnomcorder_Fight:OnEnable()
    -- Called when the addon is enabled

	GnomTEC_Gnomcorder_Fight:Print("GnomTEC_Gnomcorder_Fight Enabled")

	-- Initialize options which are propably not valid because they are new added in new versions of addon
	if (not self.db.char.characterAttributes) then
		self.db.char.characterAttributes = {}
		self.db.char.characterValues = {}		
		self.db.char.characterAttributes.MU = 10
		self.db.char.characterAttributes.ST = 10
		self.db.char.characterAttributes.WILL = 10
		self.db.char.characterAttributes.INTU = 10
		self.db.char.characterAttributes.BEW = 10		
		self.db.char.characterAttributes.GP = 10				
	end
	
	if (not self.db.char.characterAttributes.GP) then
		self.db.char.characterAttributes.GP = self.db.char.characterAttributes.gp or 0
	end

	if (not self.db.char.characterValues.RB_SUB) then
		self.db.char.characterValues.RB_SUB = self.db.char.characterValues.RB_ADD or 0
	end

	if (not self.db.char.characterValues.PA_EQUIPMENT) then
		self.db.char.characterValues.PA_EQUIPMENT = 0
	end

	if (not self.db.char.characterValues.AT_EQUIPMENT) then
		self.db.char.characterValues.AT_EQUIPMENT = 0
	end

	if (not self.db.char.characterValues.W_AT_EQUIPMENT) then
		self.db.char.characterValues.W_AT_EQUIPMENT = 0
	end

	if (not self.db.char.characterValues.SAT_EQUIPMENT) then
		self.db.char.characterValues.SAT_EQUIPMENT = 0
	end

	if (not self.db.char.characterValues.W_SAT_EQUIPMENT) then
		self.db.char.characterValues.W_SAT_EQUIPMENT = 0
	end
	
	if (not self.db.char.characterValues.RB_EQUIPMENT) then
		self.db.char.characterValues.RB_EQUIPMENT = 0
	end

	if (not self.db.char.characterValues.RS_EQUIPMENT) then
		self.db.char.characterValues.RS_EQUIPMENT = 0
	end
		
	if (not self.db.char.characterAttributes.AP) then
		self.db.char.characterAttributes.AP = 0
		self.db.char.characterValues.LP_ADD = 0
		self.db.char.characterValues.AU_ADD = 0
		self.db.char.characterValues.AN_ADD = 0
		self.db.char.characterValues.PA_ADD = 0
		self.db.char.characterValues.SAN_ADD= 0
		self.db.char.characterValues.RS_ADD = 0
		self.db.char.characterValues.RB_SUB = 0
	end
	
	

	-- clear legacy options (full clear with later versions)
	
	-- set local parameters
	GnomTEC_Gnomcorder_Fight:CheckEquipment()
	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()

	actualLP = self.db.char.characterValues.LP + self.db.char.characterValues.LP_ADD
	actualAU = self.db.char.characterValues.AU + self.db.char.characterValues.AU_ADD
		
	-- call display functions
	GnomTEC_Gnomcorder_Fight:ComputeCharacterValues()
	if (self.db.char.characterAttributes.SAVED) then
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_RESET:Show();
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Hide();	
		
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU_BUTTON:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST_BUTTON:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL_BUTTON:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU_BUTTON:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW_BUTTON:Hide()
		
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW:Show()
	else
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_RESET:Hide();
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Show();	
		
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU:Hide()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW:Hide()	

		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_MU_BUTTON:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_ST_BUTTON:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_WILL_BUTTON:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_INTU_BUTTON:Show()
		GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_VALUE_BEW_BUTTON:Show()
	
		if (self.db.char.characterAttributes.GP > 0) then
			GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Disable();	
		else
			GNOMTEC_GNOMCORDER_FIGHT_FRAME_ATTRIBUTES_SAVE:Enable();	
		end
	end
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_PLAYERMODEL:SetUnit("player")
	GNOMTEC_GNOMCORDER_FIGHT_FRAME_PLAYER_PLAYERMODEL:SetCamera(0)

	-- initialize hooks and events
	RegisterAddonMessagePrefix(CONST_ADDONMESSAGE_PREFIX);
	GnomTEC_Gnomcorder_Fight:RegisterComm(CONST_ADDONMESSAGE_PREFIX)
	
	GnomTEC_Gnomcorder_Fight:RegisterChatCommand("mea", "AttackEmote")
	GnomTEC_Gnomcorder_Fight:RegisterChatCommand("mes", "ShootEmote")
	GnomTEC_Gnomcorder_Fight:RegisterChatCommand("mei", "InspectEmote")
	
	GnomTEC_Gnomcorder_Fight:RegisterEvent("PLAYER_TARGET_CHANGED");
	GnomTEC_Gnomcorder_Fight:RegisterEvent("UNIT_INVENTORY_CHANGED");
	
	-- Add module to GnomTEC Gnomcorder
	GnomTEC_Gnomcorder:AddButton("Fight", "Fight", "Kampfsimulation", GNOMTEC_GNOMCORDER_FIGHT_FRAME, "Interface\\ICONS\\Ability_Warrior_Challange", "Interface\\ICONS\\Ability_Warrior_Challange", false, nil, GnomTEC_Gnomcorder_Fight_TimerEvent)
end

-- function called on disable of addon
function GnomTEC_Gnomcorder_Fight:OnDisable()
    -- Called when the addon is disabled
    GnomTEC_Gnomcorder_Fight:UnregisterAllEvents();
end

-- ----------------------------------------------------------------------
-- External API
-- ----------------------------------------------------------------------
